@@ -158,11 +158,12 @@ sub post_search
                     next;
                 }
 
-                $line[0] = $self->_rewriteParse($rule->{match}, $substitution, $line[0], \%rwcache);
-                if (!$line[0]) {
+                my $str = $self->_rewriteParse($rule->{match}, $substitution, $dn, \%rwcache);
+                if (!$str) {
                     $self->log(level => 'error', message => "search result rewrite \"$dn\" failed");
                     next;
                 }
+                $line[0] = "dn: $str";
 
                 my @replaced;
                 for (my $j = 1; $j < @line; $j++) {
@@ -221,25 +222,36 @@ sub pre_modify
             while (@{$listp} > 0) {
                 my $action = shift @{$listp};
                 my $attr = shift @{$listp};
+                my @values;
                 my %replaced;
 
+                $str = $self->_rewriteParse($rule->{match}, $substitution, "$action: $attr");
+                ($attr) = ($str =~ /^[^:]+: (.*)$/);
                 while (@{$listp} > 0 && ${$listp}[$0] !~ /ADD|DELETE|REPLACE/) {
-                    $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".shift @{$listp}, \%rwcache);
-                    if (!defined($str)) {
-                        $self->log(level => 'error', message => "modify rewrite \"${$dnp}\" failed");
-                        return -1;
-                    } elsif ($str) {
-                        foreach my $line (split(/\n/, $str)) {
-                            my ($rwattr, $value) = split(/: */, $line);
-                            if (!defined($replaced{$rwattr})) {
-                                @{$replaced{$rwattr}} = ();
+                    push(@values, shift @{$listp});
+                }
+
+                if (@values) {
+                    for (my $i =0; $i < @values; $i++) {
+                        $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".$values[$i], \%rwcache);
+                        if (!defined($str)) {
+                            $self->log(level => 'error', message => "modify rewrite \"${$dnp}\" failed");
+                            return -1;
+                        } elsif ($str) {
+                            foreach my $line (split(/\n/, $str)) {
+                                my ($rwattr, $value) = split(/: */, $line);
+                                if (!defined($replaced{$rwattr})) {
+                                    @{$replaced{$rwattr}} = ();
+                                }
+                                push(@{$replaced{$rwattr}}, $value);
                             }
-                            push(@{$replaced{$rwattr}}, $value);
                         }
                     }
-                }
-                foreach my $rwattr (keys %replaced) {
-                    push(@mod_list, ($action, $rwattr, @{$replaced{$rwattr}}));
+                    foreach my $rwattr (keys %replaced) {
+                        push(@mod_list, ($action, $rwattr, @{$replaced{$rwattr}}));
+                    }
+                } else {
+                    push(@mod_list, ($action, $attr));
                 }
             }
             @{$listp} = @mod_list;
@@ -287,10 +299,9 @@ sub pre_add
                 if (!defined($line[$i])) {
                     $self->log(level => 'error', message => "add rewrite \"${$dnp}\" failed");
                     return -1;
-                } elsif ($line[$i]) {
-                    ${$entryStrp}[0] = join("\n", @line)."\n";
                 }
             }
+            ${$entryStrp}[0] = join("\n", @line)."\n";
         }
     }
 
@@ -386,9 +397,9 @@ sub _checkConfig
 
     if (defined($conf->{libload})) {
         foreach my $lib (@{$conf->{libload}}) {
-            eval "require \'$lib\'";
+            eval "do \'$lib\'";
             if ($@) {
-                $self->log(level => 'alert', message => "rewrite handler require $lib: $@");
+                $self->log(level => 'alert', message => "rewrite do require $lib: $@");
                 return 1;
             }
         }
@@ -505,6 +516,10 @@ sub _ldapMap
 
     if (!$ldapmap->{ldap}) {
         $ldapmap->{ldap} = Net::LDAP->new($ldapmap->{uri});
+        if (!defined($ldapmap->{ldap})) {
+            $self->log(level => 'error', message => "Can't connect $ldapmap->{uri}");
+        }
+
         $msg = $ldapmap->{ldap}->bind($ldapmap->{binddn}, password => $ldapmap->{bindpw});
         $rc = $msg->code;
         if ($rc) {
