@@ -622,9 +622,9 @@ sub _checkConfig
 
     if (defined($conf->{libload})) {
         foreach my $lib (@{$conf->{libload}}) {
-            eval "require \'$lib\'";
+            eval "do \'$lib\'";
             if ($@) {
-                $self->log(level => 'alert', message => "storage require $lib: $@");
+                $self->log(level => 'alert', message => "storage do $lib: $@");
                 return 1;
             }
         }
@@ -654,18 +654,29 @@ sub _checkConfig
 
             # set container
             if (defined($oconf->{container}[0]->{oname})) {
-                $self->{object}{$oname}->{parent} = $oconf->{container}[0]->{oname}[0];
-                $self->{object}{$self->{object}{$oname}->{parent}}->{child} = $self->{object}{$oname};
+                my $poname = $oconf->{container}[0]->{oname}[0];
+                $self->{object}{$oname}->{parent} = $poname;
+                if (!defined($self->{object}{$poname}->{child})) {
+                    $self->{object}{$poname}->{child} = [];
+                }
+                push(@{$self->{object}{$poname}->{child}}, $self->{object}{$oname});
             } elsif (!($entry = $self->buildEntryStr($self->{suffix}, $oconf->{container}[0]))) {
                 $self->log(level => 'alert', message => "$oname object container entry is invalid");
                 return 1;
             } else {
-                $self->{object}{$oname.'_container'}->{entrystr} = $entry;
-                # normalize suffix
-                $self->{object}{$oname.'_container'}->{suffix} = lc($oconf->{container}[0]->{rdn}[0].','.$self->{suffix});
+                my $crdn = lc($oconf->{container}[0]->{rdn}[0]);
+                my $coname = $crdn.'_container';
+                if (!defined($self->{object}{$coname})) {
+                    $self->{object}{$coname}->{entrystr} = $entry;
+                    # normalize suffix
+                    $self->{object}{$coname}->{suffix} = lc("$crdn,".$self->{suffix});
+                }
                 # set child object
-                $self->{object}{$oname.'_container'}->{child} = $self->{object}{$oname};
-                $self->{object}{$oname}->{parent} = $oname.'_container';
+                if (!defined($self->{object}{$coname}->{child})) {
+                    $self->{object}{$coname}->{child} = [];
+                }
+                push(@{$self->{object}{$coname}->{child}}, $self->{object}{$oname});
+                $self->{object}{$oname}->{parent} = $coname;
             }
 
             if (defined($oconf->{entry})) {
@@ -706,26 +717,7 @@ sub _checkConfig
                 next;
             }
 
-            my $leaf_exist = 0;
-            for (my $i = 0, my $parent = $obj, my $child = $obj->{child};
-                $i < $maxLoopCount;
-                    $i++, $parent = $parent->{child}, $child = $child->{child}) {
-                if (defined($child->{conf}->{entry})) {
-                    my $entry;
-                    my ($suffix) = ($parent->{entrystr} =~ /^dn: (.*)\n/);
-                    if (!($entry = $self->buildEntryStr($suffix, $child->{conf}->{entry}[0]))) {
-                        $self->log(level => 'alert', message => "container entry is invalid");
-                        return 1;
-                    }
-                    $child->{entrystr} = $entry;
-                }
-
-                if (!defined($child->{child})) {
-                    $leaf_exist = 1;
-                    last;
-                }
-            }
-            if (!$leaf_exist) {
+            if ($self->_getTreeLevel($obj, 0) < 0) {
                 $self->log(level => 'alert', message => "depth of $oname subtree is too long");
                 return 1;
             }
@@ -735,6 +727,43 @@ sub _checkConfig
     return 0;
 }
 
+sub _getTreeLevel
+{
+    my $self = shift;
+    my ($obj, $current) = @_;
+    my $maxlevel = 0;
+
+    $current++;
+    if ($current > $maxLoopCount) {
+        return -1;
+    }
+
+    if (!defined($obj->{child})) {
+        return $current;
+    }
+
+    foreach my $child (@{$obj->{child}}) {
+        if (defined($child->{conf}->{entry})) {
+            my $entry;
+            my ($suffix) = ($obj->{entrystr} =~ /^dn: (.*)\n/);
+            if (!($entry = $self->buildEntryStr($suffix, $child->{conf}->{entry}[0]))) {
+                $self->log(level => 'alert', message => "container entry is invalid");
+                return -1;
+            }
+            $child->{entrystr} = $entry;
+        }
+
+        my $level = $self->_getTreeLevel($child, $current);
+        if ($level < 0) {
+            return -1;
+        } elsif ($level > $maxlevel) {
+            $maxlevel = $level;
+        }
+    }
+
+    return $maxlevel;
+}
+
 sub _getConnect
 {
     return 0;
@@ -772,13 +801,21 @@ sub _getObject
     }
 
     my (@rdn_list) = split(/,/, ($dn =~ /^[^,]+,(.*),?$obj->{suffix}$/i)[0]);
+    my @objs = ($obj);
     my $key;
     for (my $base = $obj->{suffix}; defined($obj->{child});
-        $obj = $obj->{child}, $base = pop(@rdn_list).','.$base) {
+        @objs = @{$obj->{child}}, $base = pop(@rdn_list).','.$base) {
         my $rc;
         my $entry;
 
-        ($rc, $key, $entry) = $self->_baseSearch($obj, $key, $base, 0, 0, 1, 0, undef);
+        for (my $i = 0; $i < @objs; $i++) {
+            ($rc, $key, $entry) = $self->_baseSearch($objs[$i], $key, $base, 0, 0, 1, 0, undef);
+            if ($entry) {
+                $obj = $objs[$i];
+                last;
+            }
+        }
+
         if (!$entry) {
             return (LDAP_NO_SUCH_OBJECT, $obj, undef);
         }
@@ -788,16 +825,15 @@ sub _getObject
     }
 
     if (defined($obj->{child})) {
-        my $child = $obj->{child};
-        if ((defined($child->{conf}->{rdn}) && $dn !~ /^$child->{conf}->{rdn}[0]=/i) ||
-            (defined($child->{entrystr}) && $child->{entrystr} !~ /^dn: $dn/i)) {
-            return (LDAP_NO_SUCH_OBJECT, $obj, undef);
+        foreach my $child (@{$obj->{child}}) {
+            if ((defined($child->{conf}->{rdn}) && $dn =~ /^$child->{conf}->{rdn}[0]=/i) ||
+                (defined($child->{entrystr}) && $child->{entrystr} =~ /^dn: $dn/i)) {
+                return (LDAP_SUCCESS, $child, $key);
+            }
         }
-
-        return (LDAP_SUCCESS, $child, $key);
-    } else {
-        return (LDAP_NO_SUCH_OBJECT, $obj, undef);
     }
+
+    return (LDAP_NO_SUCH_OBJECT, $obj, undef);
 }
 
 sub _searchObjects
@@ -842,7 +878,9 @@ sub _baseSearch
 
     if (defined($obj->{entrystr})) {
         my $entry = $obj->{entrystr};
-        if ($self->parseFilter($filter, $entry) && $scope != 1 && $sizeLim) {
+        if ($entry !~ /^dn: $base\n/i) {
+            return (LDAP_NO_SUCH_OBJECT, undef, undef);
+        } elsif ($self->parseFilter($filter, $entry) && $scope != 1 && $sizeLim) {
             return (LDAP_SUCCESS, undef, $entry);
         } else {
             return (LDAP_SUCCESS, undef, undef);
@@ -871,11 +909,8 @@ sub _childSearch
 {
     my $self = shift;
     my ($obj, $pkey, $base, $scope, $deref, $sizeLim, $timeLim, $filter) = @_;
-    my $child = undef;
     my @match_entries = ();
     my @match_keys = ();
-    my @children = ();
-    my $ckeys;
     my $rc = LDAP_SUCCESS;
 
     if ($scope == 0 || !defined($obj->{child}) || !$pkey && !defined($obj->{entrystr})) {
@@ -884,44 +919,49 @@ sub _childSearch
 
     my @entries;
     my $keys;
-    if (defined($obj->{child}->{entrystr})) {
-        my $entry = $obj->{child}->{entrystr};
-        if ($self->parseFilter($filter, $entry) && $scope != 0 && $sizeLim) {
-            push(@match_entries, $entry);
-            push(@match_keys, undef);
-        }
-    } else {
-        ($rc, $keys, @entries) = $self->_objSearch($obj->{child}, $pkey, $base, $sizeLim, $filter);
-        push(@match_entries, @entries);
-        push(@match_keys, @{$keys});
-    }
+    for (my $i = 0; $i < @{$obj->{child}}; $i++) {
+        my @children = ();
+        my $ckeys;
 
-    if ($scope == 1 || !defined($obj->{child}->{child}) || $rc) {
-        return ($rc, \@match_keys, @match_entries);
-    }
-    $sizeLim = $sizeLim - @match_entries;
-
-    if (defined($obj->{child}->{entrystr})) {
-        push(@children, $obj->{child}->{entrystr});
-        my @tmp_keys = (undef);
-        $ckeys = \@tmp_keys;
-    } else {
-        ($rc, $ckeys, @children) = $self->_objSearch($obj->{child}, $pkey, $base, -1, undef);
-        if ($rc) {
-            return ($rc, \@match_keys, @match_entries);
-        }
-    }
-
-    for (my $i = 0; $i < @{$ckeys}; $i++) {
-        my ($childdn) = ($children[$i] =~ /^dn: (.*)\n/);
-        ($rc, $keys, @entries) = $self->_childSearch($obj->{child}, ${$ckeys}[$i], $childdn, $scope, $deref, $sizeLim, $timeLim, $filter);
-        if ($rc) {
-            last;
+        if (defined($obj->{child}[$i]->{entrystr})) {
+            my $entry = $obj->{child}[$i]->{entrystr};
+            if ($self->parseFilter($filter, $entry) && $scope != 0 && $sizeLim) {
+                push(@match_entries, $entry);
+                push(@match_keys, undef);
+            }
+        } else {
+            ($rc, $keys, @entries) = $self->_objSearch($obj->{child}[$i], $pkey, $base, $sizeLim, $filter);
+            push(@match_entries, @entries);
+            push(@match_keys, @{$keys});
         }
 
-        push(@match_entries, @entries);
-        push(@match_keys, @{$keys});
-        $sizeLim = $sizeLim - @entries;
+        if ($scope == 1 || !defined($obj->{child}[$i]->{child}) || $rc) {
+            next;
+        }
+        $sizeLim = $sizeLim - @match_entries;
+
+        if (defined($obj->{child}[$i]->{entrystr})) {
+            push(@children, $obj->{child}[$i]->{entrystr});
+            my @tmp_keys = (undef);
+            $ckeys = \@tmp_keys;
+        } else {
+            ($rc, $ckeys, @children) = $self->_objSearch($obj->{child}[$i], $pkey, $base, -1, undef);
+            if ($rc) {
+                return ($rc, \@match_keys, @match_entries);
+            }
+        }
+
+        for (my $j = 0; $j < @{$ckeys}; $j++) {
+            my ($childdn) = ($children[$j] =~ /^dn: (.*)\n/);
+            ($rc, $keys, @entries) = $self->_childSearch($obj->{child}[$i], ${$ckeys}[$j], $childdn, $scope, $deref, $sizeLim, $timeLim, $filter);
+            if ($rc) {
+                return ($rc, \@match_keys, @match_entries);
+            }
+
+            push(@match_entries, @entries);
+            push(@match_keys, @{$keys});
+            $sizeLim = $sizeLim - @entries;
+        }
     }
 
     return ($rc, \@match_keys, @match_entries);
@@ -984,18 +1024,18 @@ sub _getParentRdn
     return undef;
 }
 
-sub _getStrgInfoValue
+sub _getStaticValue
 {
     my $self = shift;
-    my ($strginfo, $dn, $entryStr) = @_;
+    my ($static, $dn, $entryStr) = @_;
     my $value;
 
-    # get value of storage-specific information
-    if (defined($strginfo->{value})) {
-        if ($strginfo->{value}[0]->{type} eq 'function') {
-            eval "\$value = $strginfo->{value}[0]->{content}";
+    # get static value
+    if (defined($static->{value})) {
+        if ($static->{value}[0]->{type} eq 'function') {
+            eval "\$value = $static->{value}[0]->{content}";
         } else {
-            $value = $strginfo->{value}[0]->{content};
+            $value = $static->{value}[0]->{content};
         }
     }
 
