@@ -337,12 +337,12 @@ sub _objModify
                 if($action eq "ADD") {
                     # check whether the value already exists
                     foreach my $val (@values) {
-                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm$val$valdlm/i) {
+                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
                             $rc = LDAP_TYPE_OR_VALUE_EXISTS;
                             last DO;
                         }
                     }
-                    if ($data[$coln]) {
+                    if ($data[$coln] !~ /^ *$/) {
                         $data[$coln] = "$data[$coln]$valdlm".join($valdlm, @values);
                     } else {
                         $data[$coln] = join($valdlm, @values);
@@ -351,7 +351,7 @@ sub _objModify
                     if (@values && $values[0]) {
                         # check whether the value exists
                         foreach my $val (@values) {
-                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm$val$valdlm/i) {
+                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
                                 my $str = "$valdlm$data[$coln]$valdlm";
                                 $str =~ s/$valdlm$val$valdlm/$valdlm/i;
                                 ($data[$coln]) = ($str =~ /^$valdlm(.*)$valdlm$/);
@@ -446,7 +446,7 @@ sub _objAdd
         my @data = split(/$dlm/);
 
         # check the data correspods to the dn
-        if ("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" !~ /$valdlm$rdn_val$valdlm/i) {
+        if ("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" !~ /$valdlm *$rdn_val *$valdlm/i) {
             next;
         }
 
@@ -487,14 +487,14 @@ sub _objAdd
                     @values = @keys;
                 }
 
-                if ($data[$coln] eq '' || @values > split(/$valdlm/, $data[$coln])) {
+                if ($data[$coln] =~ /^ *$/ || @values > split(/$valdlm/, $data[$coln])) {
                     $data[$coln] = join($valdlm, @values);
                 }
             }
 
             # add storage-specific information
             foreach my $strginfo (@{$oconf->{strginfo}}) {
-                my $value = $self->_getStrgInfoValue($strginfo, $dn, $entryStr);
+                my $value = $self->_getStaticValue($strginfo, $dn, $entryStr);
 
                 if (defined($strginfo->{column})) {
                     $data[$strginfo->{column}[0]] = $value;
