@@ -414,7 +414,7 @@ sub _objModify
             if($action eq "ADD") {
                 # check whether the value already exists
                 my $vals_str = "(".join('|', @values).")";
-                if ($entry =~ /^$attr: $vals_str$/mi) {
+                if ($entry =~ /^$attr: $vals_str *$/mi) {
                     $rc = LDAP_TYPE_OR_VALUE_EXISTS;
                     last DO;
                 }
@@ -450,14 +450,14 @@ sub _objModify
             } elsif( $action eq "DELETE" ) {
                 # check whether the value exists
                 foreach $value (@values) {
-                    if ($value && !($entry =~ /^$attr: $value$/mi)) {
+                    if ($value && !($entry =~ /^$attr: $value *$/mi)) {
                         $rc = LDAP_NO_SUCH_ATTRIBUTE;
                         last DO;
                     }
                 }
 
                 if (defined($oconf->{attr}{$attr}->{column})) {
-                    if (@values && (@values > 1 || $values[0] && !($entry =~ /^$attr: $values[0]$/mi))) {
+                    if (@values && (@values > 1 || $values[0] && !($entry =~ /^$attr: $values[0] *$/mi))) {
                         $rc = LDAP_NO_SUCH_ATTRIBUTE;
                         last DO;
                     }
@@ -466,6 +466,7 @@ sub _objModify
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], decode('utf8', $sql));
 
+
                     $sth = $self->{db}->prepare($sql);
                     if (!$sth->execute) {
                         $self->log(level => 'error', message => "Deleting values by \"$sql\" failed: ".$sth->errstr);
@@ -623,7 +624,7 @@ sub _objAdd
         # get the storage-specific information
         my @si_values = (); 
         foreach my $strginfo (@{$oconf->{strginfo}}) {
-            my $value = $self->_getStrgInfoValue($strginfo, $dn, $entryStr);
+            my $value = $self->_getStaticValue($strginfo, $dn, $entryStr);
             push(@si_values, $value);
 
             if (defined($strginfo->{column})) {
@@ -776,7 +777,7 @@ sub _objDelete
             if (!defined($strginfo->{delproc})) {
                 next;
             }
-            my $value = $self->_getStrgInfoValue($strginfo, $dn);
+            my $value = $self->_getStaticValue($strginfo, $dn);
             foreach my $delproc (@{$strginfo->{delproc}}) {
 	        $sql = $delproc;
                 $sql =~ s/\%o/$key/g;
@@ -933,7 +934,9 @@ sub _getAttrValues
             } else {
                 $attrStr = $attrStr.$self->_getParentDn($aobj, $data[0])."\n";
             }
+            $attrStr =~ s/ *,/,/;
         } else{
+            $data[0] =~ s/ *$//;
             if ($data[0]) {
                 $attrStr = $attrStr."$attr: $data[0]\n";
             }
@@ -1067,6 +1070,7 @@ sub _delAttrValues
             }
         } else {
             $sql =~ s/\w*\s*=\s*'?\%a'?/1 = 1/g;
+            $sql =~ s/\w*\s*=\s*'?\%c'?/1 = 1/g;
         }
 
         # multibyte value
@@ -1167,7 +1171,7 @@ sub _filter2sql
         # attribute doesn't exit
         ${$where} = '1 = 0';
     } elsif (defined($oconf->{attr}{$attr}->{column})) {
-        ${$where} = "$oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0] $type \'$value\'";
+        ${$where} = "lower($oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]) $type lower(\'$value\')";
     } elsif (defined($oconf->{attr}{$attr}->{oname})) {
         my $attrobj = $conf->{object}{$oconf->{attr}{$attr}->{oname}[0]};
         if (defined($oconf->{attr}{$attr}->{fromtbls})) {
@@ -1175,7 +1179,7 @@ sub _filter2sql
         }
 
         ($value) = ($value =~ /^$attrobj->{rdn}[0]=([^,]*)/i);
-        ${$where} = "$attrobj->{table}[0].$attrobj->{attr}{$attrobj->{rdn}[0]}->{column}[0] $type \'$value\'";
+        ${$where} = "lower($attrobj->{table}[0].$attrobj->{attr}{$attrobj->{rdn}[0]}->{column}[0]) $type lower(\'$value\')";
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             ${$where} = "${$where} and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
@@ -1183,7 +1187,7 @@ sub _filter2sql
         if (defined($oconf->{attr}{$attr}->{fromtbls})) {
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
-        ${$where} = "$oconf->{attr}{$attr}->{selexpr}[0] $type \'$value\'";
+        ${$where} = "lower($oconf->{attr}{$attr}->{selexpr}[0]) $type lower(\'$value\')";
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             ${$where} = "${$where} and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
